@@ -7,7 +7,7 @@ import platform
 import signal
 import logging
 from PyQt5 import QtCore
-from threading import Event, Thread, current_thread, activeCount
+from threading import Event, Thread
 import webview.constant as constant
 from cefpython3 import cefpython as cef
 from uuid import uuid4
@@ -376,6 +376,50 @@ class LifeSpanHandler(object):
             logging.debug(e)
 
 
+class KeyboardHandler(object):
+    def __init__(self, browser):
+        self.browser = browser
+
+    def OnKeyEvent(self, browser, event, event_handle):
+        # MacOS系统，57.0以下版本，复制内容后，粘贴会产生两次内容。以下内容摘自66.0版本的源代码。
+        """Default implementation for keyboard shortcuts on Mac (Issue #161)."""
+        if platform.system() == "Darwin":
+            if event["modifiers"] == 128 \
+                    and event["type"] != cef.KEYEVENT_RAWKEYDOWN:
+                # Copy and paste was handled in RAWKEYDOWN, return True
+                if event["native_key_code"] in [8, 9]:
+                    return True
+            if event["modifiers"] == 128 \
+                    and event["type"] == cef.KEYEVENT_RAWKEYDOWN:
+                # Select all: command + a
+                if event["native_key_code"] == 0:
+                    browser.GetMainFrame().SelectAll()
+                    return True
+                # Copy: command + c
+                elif event["native_key_code"] == 8:
+                    browser.GetMainFrame().Copy()
+                    return True
+                # Paste: command + v
+                elif event["native_key_code"] == 9:
+                    browser.GetMainFrame().Paste()
+                    return True
+                # Cut: command + x
+                elif event["native_key_code"] == 7:
+                    browser.GetMainFrame().Cut()
+                    return True
+                # Undo: command + z
+                elif event["native_key_code"] == 6:
+                    browser.GetMainFrame().Undo()
+                    return True
+            elif event["modifiers"] == 130 \
+                    and event["type"] == cef.KEYEVENT_RAWKEYDOWN:
+                # Redo: command + shift + z
+                if event["native_key_code"] == 6:
+                    browser.GetMainFrame().Redo()
+                    return True
+        return False
+
+
 class BrowserView(QMainWindow):
     instances = {}
     menu_map = {}
@@ -542,8 +586,6 @@ class BrowserView(QMainWindow):
                 self.close_attached_window()
                 self.update_browser_info_one_by_one(increase=False)
                 event.accept()
-        # print('[ closeEvent ]', self.cid, self.uid)
-        # print('[ closeEvent ] threading.currentThread() = ', current_thread().ident)
 
     def resizeEvent(self, event):
         cef.WindowUtils.OnSize(self.winId(), 0, 0, 0)
@@ -1266,7 +1308,7 @@ def launch_main_window(uid, title, url, width, height, resizable, full_screen, m
                        quit_app_when_main_window_close=True, application_version=app_version_info,
                        enable_cef_cache=True, main_window_class_name=qt_class_name, popup_validate=None,
                        splash_screen=False, splash_text='',
-                       splash_style='font-size: 20px; font-family: Microsoft YaHei;'):
+                       splash_style='font-size: 20px; font-family: Microsoft YaHei;', *args):
     logging.debug('\r\n ---------- Application will launch ----------')
     global app_inject_obj
     global app_version_info
@@ -1327,6 +1369,16 @@ def launch_main_window(uid, title, url, width, height, resizable, full_screen, m
         'locale': 'zh-CN',
         'log_file': os.path.join(constant.debug_log_dir, time.strftime('%Y-%m-%d', time.localtime()) + '.log'),
     }
+
+    '''
+        # external_message_pump 开启，在cefpython3=57.0版本会遇到退出后弹出系统异常提示框情况。
+        if platform.system() == 'Darwin' and cef.__version__ != '66.0':
+        # 实测cefpython3 为66.0时，如果开启external_message_pump，则页面如果出现原生<select><select/>标签，则会发生闪退现象，底层原因尚未找到。
+        # 在解决问题之前，需要进行版本判断。
+        print('[ CEF LOG ] cefpython3 version = ', cef.__version__)
+        settings['external_message_pump'] = True
+    '''
+
     switches = {
         'disable-gpu': '',
         'disable-gpu-compositing': ''
@@ -1341,9 +1393,11 @@ def launch_main_window(uid, title, url, width, height, resizable, full_screen, m
 
         # framework_dir_path路径获取 研发模式和打包后的APP路径不一致
         # 研发模式下，venv目录下的路径
-        framework_dir_path_venv = os.path.abspath("./venv/lib/python" + python_version + "/site-packages/cefpython3/Chromium Embedded Framework.framework")
-        browser_subprocess_path_venv = os.path.abspath("./venv/lib/python" + python_version + "/site-packages/cefpython3/subprocess")
-        exist_venv = os.path.exists(framework_dir_path_venv)
+        if os.environ.get('VIRTUAL_ENV') and os.path.exists(os.environ['VIRTUAL_ENV']):
+            framework_dir_path_venv = os.path.abspath(
+                os.environ["VIRTUAL_ENV"] +"/lib/python" + python_version + "/site-packages/cefpython3/Chromium Embedded Framework.framework")
+            browser_subprocess_path_venv = os.path.abspath(
+                os.environ["VIRTUAL_ENV"] +"/lib/python" + python_version + "/site-packages/cefpython3/subprocess")
 
         # py2app 生产编译模式下，Contents/Resources下的路径
         framework_dir_path_py2app = os.path.abspath("./lib/python" + python_version + "/cefpython3/Chromium Embedded Framework.framework")
@@ -1360,15 +1414,19 @@ def launch_main_window(uid, title, url, width, height, resizable, full_screen, m
         exist_pyinstaller = os.path.exists(framework_dir_path_pyinstaller)
 
         if exist_py2app:
+            print("[ CEF LOG ] 当前应用为生产模式，使用py2app打包构建")
             framework_dir_path = framework_dir_path_py2app
             browser_subprocess_path = browser_subprocess_path_py2app
         elif exist_system:
+            print("[ CEF LOG ] 当前应用为研发模式，使用py2app -a 模式进行调试")
             framework_dir_path = framework_dir_path_system
             browser_subprocess_path = browser_subprocess_path_system
         elif exist_pyinstaller:
+            print("[ CEF LOG ] 当前应用为生产模式，使用pyinstaller打包构建")
             framework_dir_path = framework_dir_path_pyinstaller
             browser_subprocess_path = browser_subprocess_path_pyinstaller
         else:
+            print("[ CEF LOG ] 当前应用为研发模式，使用研发环境venv目录")
             framework_dir_path = framework_dir_path_venv
             browser_subprocess_path = browser_subprocess_path_venv
 
@@ -1395,6 +1453,7 @@ def launch_main_window(uid, title, url, width, height, resizable, full_screen, m
 def set_client_handler(uid, payload, cid, browser, popup_validate):
     BrowserView.instances[uid].view.SetClientHandler(LoadHandler(uid, payload, cid, browser))
     BrowserView.instances[uid].view.SetClientHandler(LifeSpanHandler(popup_validate))
+    BrowserView.instances[uid].view.SetClientHandler(KeyboardHandler(browser))
 
 
 def set_javascript_bindings(uid, inject_obj=app_inject_obj, application_version=app_version_info, payload={}):
